import hashlib, binascii, os
import flaschengeist.modules as modules

class AuthPlain(modules.Auth):
    def login(self, user, password):
        if not user:
            return False
        if 'password' in user.attributes:
            return self.__verify_password(user.attributes['password'].value, password)
        return False

    def __hash_password(self, password):
        salt = hashlib.sha256(os.urandom(60)).hexdigest().encode('ascii')
        pwdhash = hashlib.pbkdf2_hmac('sha3-512', password.encode('utf-8'), salt, 100000)
        pwdhash = binascii.hexlify(pwdhash)
        return (salt + pwdhash).decode('ascii')

    def __verify_password(self, stored_password, provided_password):
        salt = stored_password[:64]
        stored_password = stored_password[64:]
        pwdhash = hashlib.pbkdf2_hmac('sha3-512', provided_password.encode('utf-8'),
                                  salt.encode('ascii'), 100000)
        pwdhash = binascii.hexlify(pwdhash).decode('ascii')
        return pwdhash == stored_password
